function [stat] = freqstatistics(cfg, varargin)

% FREQSTATISTICS computes the probability for a given null-hypothesis using
% a parametric statistical test or using a non-parametric randomization test.
%
% Use as
%   [stat] = freqstatistics(cfg, freq1, freq2, ...)
% where the input data is the result from FREQANALYSIS, FREQDESCRIPTIVES
% or from FREQGRANDAVERAGE.
%
% The configuration can contain the following options for data selection
%   cfg.channel     = Nx1 cell-array with selection of channels (default = 'all'),
%                     see CHANNELSELECTION for details
%   cfg.latency     = [begin end] in seconds or 'all' (default = 'all')
%   cfg.frequency   = [begin end], can be 'all'       (default = 'all')
%   cfg.avgoverchan = 'yes' or 'no'                   (default = 'no')
%   cfg.avgovertime = 'yes' or 'no'                   (default = 'no')
%   cfg.avgoverfreq = 'yes' or 'no'                   (default = 'no')
%   cfg.parameter   = string                          (default = 'powspctrm')
%
% Furthermore, the configuration should contain
%   cfg.method       = different methods for calculating the probability of the null-hypothesis,
%                    'montecarlo' use a non-parametric randomization test to get a Monte-Carlo estimate of the probability,
%                    'analytic'   use a parametric test that results in analytic probability,
%                    'stats'      use a parametric test from the Matlab statistics toolbox,
%                    'glm'        use a general linear model approach.
%
% The other cfg options depend on the method that you select. You
% should read the help of the respective subfunction STATISTICS_XXX
% for the corresponding configuration options and for a detailed
% explanation of each method.
%
% See also FREQANALYSIS, FREQDESCRIPTIVES, FREQGRANDAVERAGE

% This function depends on STATISTICS_WRAPPER
%
% TODO change cfg.frequency in all functions to cfg.foi or cfg.foilim

% Copyright (C) 2005-2006, Robert Oostenveld
%
% $Log: freqstatistics.m,v $
% Revision 1.11  2006/11/27 15:38:20  roboos
% implemented support for cfg.parameter, by locally renaming the field in the data structure
%
% Revision 1.10  2006/10/19 15:05:51  roboos
% updated documentation
%
% Revision 1.9  2006/10/04 07:10:07  roboos
% updated documentation
%
% Revision 1.8  2006/07/12 09:18:17  roboos
% improved documentation
%
% Revision 1.7  2006/06/20 16:25:58  ingnie
% updated documentation
%
% Revision 1.6  2006/06/20 12:57:26  roboos
% updated documentation
%
% Revision 1.5  2006/06/13 14:48:09  ingnie
% updated documentation

% the low-level data selection function does not know how to deal with other parameters, so work around it
if isfield(cfg, 'parameter') && ~strcmp(cfg, 'powspctrm')
  fprintf('renaming parameter ''%s'' into ''powspctrm''\n', cfg.parameter);
  for i=1:length(varargin)
   dat         = getsubfield(varargin{i}, cfg.parameter);
   varargin{i} = rmsubfield (varargin{i}, cfg.parameter);
   varargin{i} = setsubfield(varargin{i}, 'powspctrm', dat);
  end
end

% call the general function
[stat] = statistics_wrapper(cfg, varargin{:});

