function [source] = sourcedescriptives(cfg, source)

% SOURCEDESCRIPTIVES computes descriptive parameters of the beamformer source
% analysis results.
%
% Use as: 
%   [source] = sourcedescriptives(cfg, source)
%
% where cfg is a structure with the configuration details and source is the
% result from a beamformer source estimation. The configuration can contain
%   cfg.transform  = string describing the transformation (default is [])
%
% The nai, i.e. neural activity index (power divided by projected noise),
% is computed prior to applying the optional transformation.
%
% Subsequently, the transformation is applied on the power and on the projected
% noise using "feval". A usefull transformation is for example 'log' or 'log10'.
%
% If repeated trials are present that have undergone some sort of
% resampling (i.e. jackknife, bootstrap, singletrial or rawtrial), the mean,
% variance and standard error of mean will be computed for all source
% parameters. This is done after applying the optional transformation
% on the power and projected noise.
%
% See also SOURCEANALYSIS, SOURCESTATISTICS

% Undocumented local options:
% cfg.cohmethod = 'dics', 'svdfft' or 'canonical'
% cfg.feedback
% cfg.supmethod
% cfg.resolutionmatrix = yes/no

% Copyright (C) 2004-2006, Robert Oostenveld
%
% $Log: sourcedescriptives.m,v $
% Revision 1.29  2006/07/04 16:04:50  roboos
% renamed option 'jacknife' into 'jackknife' for consistency, maintain backward compatibility with cfgs and old data
%
% Revision 1.28  2006/06/27 12:01:22  roboos
% removed cfg.resolutionfield and cfg.pointspread (wer noit yet implemented), implemented cfg.resolutionmatrix=yes/no (only for pcc with no refdip/chan)
%
% Revision 1.27  2006/05/11 15:18:11  jansch
% removed the hard-coded indexing in the case of computing coherence from mom
%
% Revision 1.26  2006/05/04 15:41:40  roboos
% cleaned up part of the pcc code
% added svdfft rotation of the filters and/or leadfields if present
%
% Revision 1.25  2006/05/03 15:10:51  roboos
% added a comment
%
% Revision 1.24  2006/04/27 07:26:07  jansch
% added support for singletrial csd-computation, if applicable
%
% Revision 1.23  2006/04/20 09:58:34  roboos
% updated documentation
%
% Revision 1.22  2006/04/12 13:45:26  jansch
% implemented the option cfg.singletrial: in the case of a pcc-beamed input,
% single trial power-estimates are outputted in the field .trial
%
% Revision 1.21  2006/01/30 17:10:33  roboos
% replaced fprintf with progress() indicator
%
% Revision 1.20  2006/01/24 14:31:05  roboos
% added cfg.cohmethod as undocumented option (not in main help, but just below it)
%
% Revision 1.19  2005/10/18 12:21:27  roboos
% fixed bug in power estimation from noise csd
% removed some obsolete code (was already commented out)
%
% Revision 1.18  2005/09/16 07:37:20  jansch
% in case of svd, only do svd on the real part
%
% Revision 1.17  2005/09/05 06:39:20  jansch
% experimental inclusion of partialisation in svdfft-option. no guarantees
% for success!
%
% Revision 1.16  2005/09/01 12:52:07  jansch
% included possibility to do svd on the single-trial ffts.
%
% Revision 1.15  2005/06/29 12:46:29  roboos
% the following changes were done on a large number of files at the same time, not all of them may apply to this specific file
% - added try-catch around the inclusion of input data configuration
% - moved cfg.version, cfg.previous and the assignment of the output cfg to the end
% - changed help comments around the configuration handling
% - some changes in whitespace
%
% Revision 1.14  2005/06/15 07:10:40  roboos
% fixed obvious typing mistake that caused error for mom field
%
% Revision 1.13  2005/06/13 08:04:20  roboos
% fixed a important bug in the computation of jackknife estimate of variance: the estimate was sqrt(N) times too small (i.e. it was the SEM instead of VAR)
% made the computation of the variance more readible by separating the bias term from the 1/(N-1) division
%
% Revision 1.12  2005/05/20 16:50:19  roboos
% added normalisation of the CSD matrix computed from the fourier spectra
%
% Revision 1.11  2005/05/19 16:15:19  roboos
% source does not always contain an avg, do conditional isfield for source.avg.csdlabel
%
% Revision 1.10  2005/05/17 17:58:28  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
% added preliminary implementation for postprocessing of pcc source csd matrices
%
% Revision 1.9  2005/03/03 13:57:29  roboos
% implemented pseudovalue resampling, i.e. a combination of jackknife and the complete average
%
% Revision 1.8  2004/10/27 13:13:02  roboos
% included source.trial.noise and source.trial.nai in the computation of avg, var and sem
%
% Revision 1.7  2004/10/13 15:54:05  roboos
% implemented trandformation on pow and noise (cfg.transform) prior to computing descriptive stats
%
% Revision 1.6  2004/09/22 14:09:52  roboos
% renamed method=randomized into method=randomization, with backward compatibility
%
% Revision 1.5  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%
% Revision 1.4  2004/08/04 16:10:41  roboos
% fixed bug in randomized data when different number of trials, added support for permuted resamplings
%
% Revision 1.3  2004/05/24 14:53:56  roberto
% fixed a typing mistake in the help
%
% Revision 1.2  2004/02/10 13:02:26  roberto
% fixed (obvious) bug due to not looping over trials
%
% Revision 1.1  2004/01/21 13:18:56  roberto
% new function that computes nai for all sourcereconstructions and
% var/avg/sem for sourcereconstructions that have been jackknife or
% otherwise resampled over trials
%

% set the defaults
if ~isfield(cfg, 'transform'),        cfg.transform        = [];        end
if ~isfield(cfg, 'cohmethod'),        cfg.cohmethod        = 'dics';    end
if ~isfield(cfg, 'feedback'),         cfg.feedback         = 'textbar'; end
if ~isfield(cfg, 'supmethod'),        cfg.supmethod        = 'none';    end
if ~isfield(cfg, 'singletrial'),      cfg.singletrial      = 'no';      end
if ~isfield(cfg, 'resolutionmatrix'), cfg.resolutionmatrix = 'no';      end

% this is required for backward compatibility with the old sourceanalysis
if isfield(source, 'method') && strcmp(source.method, 'randomized')
  source.method = 'randomization';
elseif isfield(source, 'method') && strcmp(source.method, 'permuted')
  source.method = 'permutation';
elseif isfield(source, 'method') && strcmp(source.method, 'jacknife')
  source.method = 'jackknife';
end

% do a validity check on the input data and specified options
if strcmp(cfg.resolutionmatrix, 'yes')
  if ~isfield(source.avg, 'filter')
    error('The computation of the resolution matrix requires keepfilter=''yes'' in sourceanalysis.');
  elseif ~isfield(source, 'leadfield')
    error('The computation of the resolution matrix requires keepleadfield=''yes'' in sourceanalysis.');
  end
end

if isfield(source, 'avg') && isfield(source.avg, 'csdlabel') 
  % the source reconstruction was computed using the pcc beamformer
  Ndipole    = length(source.inside) + length(source.outside);
  dipsel     = match_str(source.avg.csdlabel, 'scandip');
  refchansel = match_str(source.avg.csdlabel, 'refchan');
  refdipsel  = match_str(source.avg.csdlabel, 'refdip');
  supchansel = match_str(source.avg.csdlabel, 'supchan');
  supdipsel  = match_str(source.avg.csdlabel, 'supdip');

  % cannot handle reference channels and reference dipoles simultaneously
  if length(refchansel)>0 && length(refdipsel)>0
    error('cannot simultaneously handle reference channels and reference dipole');
  end

  % these are only used to count the number of reference/suppression dipoles and channels
  refsel = [refdipsel refchansel];
  supsel = [supdipsel supchansel];

  if strcmp(cfg.cohmethod, 'svdfft'),
    source.avg.pow = zeros(Ndipole, 1);
    source.avg.coh = zeros(Ndipole, 1);
    progress('init', cfg.feedback, 'performing svd on dipole moment');
    for diplop=1:length(source.inside)
      progress(diplop/length(source.inside), 'performing svd on dipole moment %d/%d\n', diplop, length(source.inside));
      i       = source.inside(diplop);
      mom     = source.avg.mom{i}(dipsel,     :); 
      ref     = source.avg.mom{i}(refdipsel,  :); 
      sup     = source.avg.mom{i}(supdipsel,  :);
      refchan = source.avg.mom{i}(refchansel, :);
      supchan = source.avg.mom{i}(supchansel, :);
      % compute the projection of the scanning dipole along the direction of the dominant amplitude
      if length(dipsel)>1,    [mom, rmom] = svdfft(mom, 1, source.cumtapcnt); else rmom = []; end
      % compute the projection of the reference dipole along the direction of the dominant amplitude
      if length(refdipsel)>1, [ref, rref] = svdfft(ref, 1, source.cumtapcnt); else rref = []; end
      % compute the projection of the supression dipole along the direction of the dominant amplitude
      if length(supdipsel)>1, [sup, rsup] = svdfft(sup, 1, source.cumtapcnt); else rsup = []; end
      % compute voxel-level cross-spectral density
      dat = cat(1, mom, ref, sup, refchan, supchan);
      
      momindx = [1:size(mom,1)];
      if ~isempty(ref),     refindx = [(size(mom,1)+1):(size(mom,1)+size(ref,1))];                                 end
      if ~isempty(refchan), refindx = [(size(mom,1)+size(ref,1)+size(sup,1)+1):...
                                       (size(mom,1)+size(ref,1)+size(sup,1)+size(refchan,1))];                     end
      if ~isempty(sup),      supdipindx  = [(size(mom,1)+size(ref,1)+1):...
                                           (size(mom,1)+size(ref,1)+size(sup,1))];                                 end
      if ~isempty(supchan),  supchanindx = [(size(mom,1)+size(ref,1)+size(sup,1)+size(refchan,1)+1):...
                                           (size(mom,1)+size(ref,1)+size(sup,1)+size(refchan,1)+size(supchan,1))]; end
      %if all(source.cumtapcnt==source.cumtapcnt(1)),
      %  csd = (dat * dat'); 
      %  csd = csd./sum(source.cumtapcnt); %normalize for the total number of tapers 
      %else
        sumtapcnt = cumsum([0;source.cumtapcnt(:)]);
        csd       = zeros(size(dat,1),size(dat,1));
        for j = 1:length(source.cumtapcnt)
          tmpcsd = [dat(:, sumtapcnt(j)+1:sumtapcnt(j+1)) * dat(:, sumtapcnt(j)+1:sumtapcnt(j+1))']./source.cumtapcnt(j);
          csd    = csd + tmpcsd;
          %compute singletrial power and csd, and put it in the trial-field
          if strcmp(cfg.singletrial, 'yes'),
            if diplop == 1, %allocate some memory, can be done only now, since the dimensionality was not known before
              source.trial(j).pow = zeros(Ndipole, size(tmpcsd,1)                );
              if prod(size(tmpcsd))>1, source.trial(j).csd = zeros(Ndipole, size(tmpcsd,1), size(tmpcsd,2)); end
            end
            %this is non-partialized power
            source.trial(j).pow(i,:)   = transpose(abs(diag(tmpcsd)));
            %this is normal csd, and only create it, if it represents more than just a power-spectrum
            if prod(size(tmpcsd))>1, source.trial(j).csd(i,:,:) = tmpcsd; end
          end
        end
        csd = csd./length(source.cumtapcnt); %normalize for the number of trials, weighing for the number of tapers has already been done
      %end
      source.avg.csd{i} = csd; 
      source.avg.mom{i} = dat;
      if prod(size(csd))>1,
        if strcmp(cfg.supmethod, 'none'),
          source.avg.coh(i) = (csd(momindx,refindx).^2)./(abs(csd(momindx,momindx)).*abs(csd(refindx,refindx)));
        elseif strcmp(cfg.supmethod, 'chan_dip') && size(dat,1)==4,
          pcsd              = csd([momindx refindx],[momindx refindx]) - (csd([momindx refindx],[supdipindx supchanindx])*inv(csd([supdipindx supchanindx],[supdipindx supchanindx]))*csd([supdipindx supchanindx],[momindx refindx]));
          source.avg.coh(i) = (pcsd(momindx,refindx).^2)./(abs(pcsd(momindx,momindx)).*abs(pcsd(refindx,refindx)));
        elseif strcmp(cfg.supmethod, 'chan') && ~isempty(supchan) && ~isempty(supdip),
          pcsd              = csd([momindx refindx supdipindx],[momindx refindx supdipindx]) - (csd([momindx refindx supdipindx],supchanindx)*inv(csd(supchanindx,supchanindx))*csd(supchanindx,[momindx refindx supdipindx]));
          source.avg.coh(i) = (pcsd(momindx,refindx).^2)./(abs(pcsd(momindx,momindx)).*abs(pcsd(refindx,refindx)));
        elseif strcmp(cfg.supmethod, 'chan') && ~isempty(supchan) && isempty(supdip),
          pcsd              = csd([momindx refindx],[momindx refindx]) - (csd([momindx refindx],supchanindx)*inv(csd(supchanindx,supchanindx))*csd(supchanindx,[momindx refindx]));
          source.avg.coh(i) = (pcsd(momindx,refindx).^2)./(abs(pcsd(momindx,momindx)).*abs(pcsd(refindx,refindx)));
        elseif strcmp(cfg.supmethod, 'dip') && ~isempty(supdip),
          pcsd              = csd([momindx refindx],[momindx refindx]) - (csd([momindx refindx],supdipindx)*inv(csd(supdipindx,supdipindx))*csd(supdipindx,[momindx refindx]));
          source.avg.coh(i) = (pcsd(momindx,refindx).^2)./((pcsd(momindx,momindx)).*(pcsd(refindx,refindx)));
        else
          error('what you want to do does not seem to make sense');
        end
        %FIXME: the preceding code has to be tested and be rewritten
      end
      source.avg.pow(i) = abs(csd(1,1));
      if isfield(source, 'leadfield')
        % apply the same rotation on the leadfield
        source.leadfield{i} = source.leadfield{i} * rmom';
      end
      if isfield(source.avg, 'filter')
        % apply the same rotation on the filter
        scandipfilt = rmom * source.avg.filter{i}(dipsel,:);
        refdipfilt  = rref * source.avg.filter{i}(refdipsel,:);
        supdipfilt  = rsup * source.avg.filter{i}(supdipsel,:);
        refchanfilt =        source.avg.filter{i}(refchansel,:);
        supchanfilt =        source.avg.filter{i}(supchansel,:);
        source.avg.filter{i} = cat(1, scandipfilt, refdipfilt, supdipfilt, refchanfilt, supchanfilt);
      end
    end % for diplop
    progress('close');

    % remember what the interpretation is of all CSD output components
    scandiplabel = repmat({'scandip'}, 1, 1);                    % only one dipole orientation remains
    refdiplabel  = repmat({'refdip'},  1, length(refdipsel)>0);  % for svdfft at max. only one dipole orientation remains
    supdiplabel  = repmat({'supdip'},  1, length(supdipsel)>0);  % for svdfft at max. only one dipole orientation remains
    refchanlabel = repmat({'refchan'}, 1, length(refchansel));
    supchanlabel = repmat({'supchan'}, 1, length(supchansel));
    % concatenate all the labels
    source.avg.csdlabel = cat(2, scandiplabel, refdiplabel, supdiplabel, refchanlabel, supchanlabel);

    if isfield(source.avg, 'noisecsd'),
      % the noisecsd did not get the same svdfft treatment, therefore remove it
      source.avg = rmfield(source.avg, 'noisecsd');
    end; 

  elseif strcmp(cfg.cohmethod, 'canonical') && ~isfield(source.avg, 'coh') && ~isempty(refsel) && ~isempty(supsel)
    % compute power and canonical coherence, with partialisation
    fprintf('computing canonical coherence for pcc beamformer with partialisation\n');
    if strcmp(cfg.supmethod, 'chan_dip'),
      supindx = [supdipsel supchansel];
    elseif strcmp(cfg.supmethod, 'chan'),
      supindx = supchansel;
    elseif strcmp(cfg.supmethod, 'dip'),
      supindx = supdipsel;
    elseif strcmp(cfg.supmethod, 'none'),
      supindx = [];
    end
    scnindx = setdiff(1:size(source.avg.csd{source.inside(1)},1),supindx);
    [dum1, dipsel, dum2] = intersect(scnindx, dipsel);
    [dum1, refsel, dum2] = intersect(scnindx, refsel);
    source.avg.pow = zeros(Ndipole, 1);
    source.avg.coh = zeros(Ndipole, 1);
    for i=source.inside(:)'
      csd  = source.avg.csd{i};
      % compute partialised cross-spectrum
      pcsd = csd(scnindx, scnindx) - csd(scnindx, supindx) * pinv(csd(supindx, supindx)) * csd(supindx, scnindx);
      % compute partial-canonical-coherence
      ccoh = diag(cancorr(pcsd, dipsel, refsel));
      [cmax, indmax] = max(ccoh);
      source.avg.coh(i) = ccoh(indmax);
    end
    source.avg.pow(source.outside) = nan;
    source.avg.coh(source.outside) = nan;

  elseif strcmp(cfg.cohmethod, 'canonical') && ~isfield(source.avg, 'coh') && ~isempty(refsel) && isempty(supsel)
    % compute power and canonical coherence, no partialisation
    fprintf('computing canonical coherence for pcc beamformer\n');
    source.avg.pow = zeros(Ndipole, 1);
    source.avg.coh = zeros(Ndipole, 1);
    for i=source.inside(:)'
      [ccoh, c2, v1, v2] = cancorr(source.avg.csd{i}, dipsel, refsel);
      [cmax, indmax]    = max(ccoh);
      %source.avg.coh(i) = ccoh(indmax);
    end
    source.avg.pow(source.outside) = nan;
    source.avg.coh(source.outside) = nan;

  elseif strcmp(cfg.cohmethod, 'dics') && ~isfield(source.avg, 'coh') && ~isempty(refsel)
    % compute power and coherence on Joachim Gross' way
    fprintf('computing power and coherence for pcc beamformer\n');
    source.avg.pow = zeros(Ndipole, 1);
    source.avg.coh = zeros(Ndipole, 1);
    for i=source.inside(:)'
      Pd  = lambda1(source.avg.csd{i}(dipsel,dipsel));
      Pr  = lambda1(source.avg.csd{i}(refsel,refsel));
      Cdr = lambda1(source.avg.csd{i}(dipsel,refsel));
      source.avg.pow(i) = Pd;
      source.avg.coh(i) = abs(Cdr).^2 ./ (Pd*Pr);
    end
    source.avg.pow(source.outside) = nan;
    source.avg.coh(source.outside) = nan;

  elseif ~isfield(source.avg, 'pow')
    fprintf('computing power for pcc beamformer\n');
    % compute power on Joachim Gross' way
    source.avg.pow = zeros(Ndipole, 1);
    for i=source.inside(:)'
      source.avg.pow(i) = lambda1(source.avg.csd{i});
    end
    source.avg.pow(source.outside) = nan;
  end % computing power and coherence

  if ~isfield(source.avg, 'noise') && isfield(source.avg, 'noisecsd')
    % compute power of projected noise
    fprintf('computing power of projected noise for pcc beamformer\n');
    source.avg.noise = zeros(Ndipole, 1);
    % compute projected noise on Joachim Gross' way
    for i=source.inside(:)'
      source.avg.noise(i) = lambda1(source.avg.noisecsd{i});
    end
    source.avg.noise(source.outside) = nan;
  end % computing projected noise

end % dealing with pcc input

if isfield(source, 'avg') && isfield(source.avg, 'pow') && isfield(source.avg, 'noise')
  % compute the neural activity index for the average
  source.avg.nai = source.avg.pow ./ source.avg.noise;
end

if isfield(source, 'trial') && isfield(source.trial, 'pow') && isfield(source.trial, 'noise')
  % compute the neural activity index for the trials
  ntrials = length(source.trial);
  for trlop=1:ntrials
    source.trial(trlop).nai = source.trial(trlop).pow ./ source.trial(trlop).noise;
  end
end

if strcmp(source.method, 'randomization') || strcmp(source.method, 'permutation')
  % compute the neural activity index for the two randomized conditions
  source.avgA.nai = source.avgA.pow ./ source.avgA.noise;
  source.avgB.nai = source.avgB.pow ./ source.avgB.noise;
  for trlop=1:length(source.trialA)
    source.trialA(trlop).nai = source.trialA(trlop).pow ./ source.trialA(trlop).noise;
  end
  for trlop=1:length(source.trialB)
    source.trialB(trlop).nai = source.trialB(trlop).pow ./ source.trialB(trlop).noise;
  end
end

if ~isempty(cfg.transform)
  fprintf('applying %s transformation on the power and projected noise\n', cfg.transform);
  % apply the specified transformation on the power
  if isfield(source, 'avg'   ) && isfield(source.avg   , 'pow'), source.avg .pow = feval(cfg.transform, source.avg .pow); end
  if isfield(source, 'avgA'  ) && isfield(source.avgA  , 'pow'), source.avgA.pow = feval(cfg.transform, source.avgA.pow); end
  if isfield(source, 'avgB'  ) && isfield(source.avgB  , 'pow'), source.avgB.pow = feval(cfg.transform, source.avgB.pow); end
  if isfield(source, 'trial' ) && isfield(source.trial , 'pow'), for i=1:length(source.trial ), source.trial (i).pow = feval(cfg.transform, source.trial (i).pow); end; end
  if isfield(source, 'trialA') && isfield(source.trialA, 'pow'), for i=1:length(source.trialA), source.trialA(i).pow = feval(cfg.transform, source.trialA(i).pow); end; end
  if isfield(source, 'trialB') && isfield(source.trialB, 'pow'), for i=1:length(source.trialB), source.trialB(i).pow = feval(cfg.transform, source.trialB(i).pow); end; end
  % apply the specified transformation on the projected noise
  if isfield(source, 'avg'   ) && isfield(source.avg   , 'noise'), source.avg .noise = feval(cfg.transform, source.avg .noise); end
  if isfield(source, 'avgA'  ) && isfield(source.avgA  , 'noise'), source.avgA.noise = feval(cfg.transform, source.avgA.noise); end
  if isfield(source, 'avgB'  ) && isfield(source.avgB  , 'noise'), source.avgB.noise = feval(cfg.transform, source.avgB.noise); end
  if isfield(source, 'trial' ) && isfield(source.trial , 'noise'), for i=1:length(source.trial ), source.trial (i).noise = feval(cfg.transform, source.trial (i).noise); end; end
  if isfield(source, 'trialA') && isfield(source.trialA, 'noise'), for i=1:length(source.trialA), source.trialA(i).noise = feval(cfg.transform, source.trialA(i).noise); end; end
  if isfield(source, 'trialB') && isfield(source.trialB, 'noise'), for i=1:length(source.trialB), source.trialB(i).noise = feval(cfg.transform, source.trialB(i).noise); end; end
end

if strcmp(source.method, 'pseudovalue')
  % compute the pseudovalues for the beamformer output 
  avg = source.trial(1);		% the first is the complete average
  Ntrials = length(source.trial)-1;	% the remaining are the leave-one-out averages
  pseudoval = [];
  if isfield(source.trial, 'pow')
    allavg = getfield(avg, 'pow');
    for i=1:Ntrials
      thisavg = getfield(source.trial(i+1), 'pow');
      thisval = Ntrials*allavg - (Ntrials-1)*thisavg;
      pseudoval(i).pow = thisval;
    end
  end
  if isfield(source.trial, 'coh')
    allavg = getfield(avg, 'coh');
    for i=1:Ntrials
      thisavg = getfield(source.trial(i+1), 'coh');
      thisval = Ntrials*allavg - (Ntrials-1)*thisavg;
      pseudoval(i).coh = thisval;
    end
  end
  if isfield(source.trial, 'nai')
    allavg = getfield(avg, 'nai');
    for i=1:Ntrials
      thisavg = getfield(source.trial(i+1), 'nai');
      thisval = Ntrials*allavg - (Ntrials-1)*thisavg;
      pseudoval(i).nai = thisval;
    end
  end
  if isfield(source.trial, 'noise')
    allavg = getfield(avg, 'noise');
    for i=1:Ntrials
      thisavg = getfield(source.trial(i+1), 'noise');
      thisval = Ntrials*allavg - (Ntrials-1)*thisavg;
      pseudoval(i).noise = thisval;
    end
  end
  % store the pseudovalues instead of the original values
  source.trial = pseudoval;
end

if strcmp(source.method, 'jackknife') || strcmp(source.method, 'bootstrap') || strcmp(source.method, 'pseudovalue') || strcmp(source.method, 'singletrial') || strcmp(source.method, 'rawtrial') 
  % compute descriptive statistics (mean, var, sem) for multiple trial data
  % compute these for as many source parameters as possible

  % for convenience copy the trials out of the source structure
  dip = source.trial;

  % determine the (original) number of trials in the data
  if strcmp(source.method, 'bootstrap')
    Ntrials = size(source.resample, 2);
  else
    Ntrials = length(source.trial);
  end
  fprintf('original data contained %d trials\n', Ntrials);

  % allocate memory for all elements in the dipole structure
  sumdip = [];
  if isfield(dip(1), 'var'),    sumdip.var    = zeros(size(dip(1).var  )); end
  if isfield(dip(1), 'pow'),    sumdip.pow    = zeros(size(dip(1).pow  )); end
  if isfield(dip(1), 'coh'),    sumdip.coh    = zeros(size(dip(1).coh  )); end
  if isfield(dip(1), 'rv'),     sumdip.rv     = zeros(size(dip(1).rv   )); end
  if isfield(dip(1), 'noise'),  sumdip.noise  = zeros(size(dip(1).noise)); end
  if isfield(dip(1), 'nai'),    sumdip.nai    = zeros(size(dip(1).nai  )); end
  sqrdip = [];
  if isfield(dip(1), 'var'),    sqrdip.var    = zeros(size(dip(1).var  )); end
  if isfield(dip(1), 'pow'),    sqrdip.pow    = zeros(size(dip(1).pow  )); end
  if isfield(dip(1), 'coh'),    sqrdip.coh    = zeros(size(dip(1).coh  )); end
  if isfield(dip(1), 'rv'),     sqrdip.rv     = zeros(size(dip(1).rv   )); end
  if isfield(dip(1), 'noise'),  sqrdip.noise  = zeros(size(dip(1).noise)); end
  if isfield(dip(1), 'nai'),    sqrdip.nai    = zeros(size(dip(1).nai  )); end
  if isfield(dip(1), 'mom')
    sumdip.mom = cell(size(dip(1).mom));
    sqrdip.mom = cell(size(dip(1).mom));
    for i=1:length(dip(1).mom)
      sumdip.mom{i} = zeros(size(dip(1).mom{i}));
      sqrdip.mom{i} = zeros(size(dip(1).mom{i}));
    end
  end
  if isfield(dip(1), 'csd')
    sumdip.csd = cell(size(dip(1).csd));
    sqrdip.csd = cell(size(dip(1).csd));
    for i=1:length(dip(1).csd)
      sumdip.csd{i} = zeros(size(dip(1).csd{i}));
      sqrdip.csd{i} = zeros(size(dip(1).csd{i}));
    end
  end

  for trial=1:length(dip)
    % compute the sum of all values
    if isfield(dip(trial), 'var'),    sumdip.var   = sumdip.var    + dip(trial).var;    end
    if isfield(dip(trial), 'pow'),    sumdip.pow   = sumdip.pow    + dip(trial).pow;    end
    if isfield(dip(trial), 'coh'),    sumdip.coh   = sumdip.coh    + dip(trial).coh;    end
    if isfield(dip(trial), 'rv'),     sumdip.rv    = sumdip.rv     + dip(trial).rv;     end
    if isfield(dip(trial), 'noise'),  sumdip.noise = sumdip.noise  + dip(trial).noise;  end
    if isfield(dip(trial), 'nai'),    sumdip.nai   = sumdip.nai    + dip(trial).nai;    end
    % compute the sum of squared values
    if isfield(dip(trial), 'var'),    sqrdip.var    = sqrdip.var   + (dip(trial).var  ).^2; end
    if isfield(dip(trial), 'pow'),    sqrdip.pow    = sqrdip.pow   + (dip(trial).pow  ).^2; end
    if isfield(dip(trial), 'coh'),    sqrdip.coh    = sqrdip.coh   + (dip(trial).coh  ).^2; end
    if isfield(dip(trial), 'rv'),     sqrdip.rv     = sqrdip.rv    + (dip(trial).rv   ).^2; end
    if isfield(dip(trial), 'noise'),  sqrdip.noise  = sqrdip.noise + (dip(trial).noise).^2; end
    if isfield(dip(trial), 'nai'),    sqrdip.nai    = sqrdip.nai   + (dip(trial).nai  ).^2; end
    % do the same for the cell array with mom
    if isfield(dip(trial), 'mom')
      for i=1:length(dip(1).mom)
        sumdip.mom{i} = sumdip.mom{i} +  dip(trial).mom{i};
        sqrdip.mom{i} = sqrdip.mom{i} + (dip(trial).mom{i}).^2;
      end
    end
    % do the same for the cell array with csd
    if isfield(dip(trial), 'csd')
      for i=1:length(dip(1).csd)
        sumdip.csd{i} = sumdip.csd{i} +  dip(trial).csd{i};
        sqrdip.csd{i} = sqrdip.csd{i} + (dip(trial).csd{i}).^2;
      end
    end
  end

  % compute the mean over all repetitions
  if isfield(sumdip, 'var'),    dipmean.var    = sumdip.var   / length(dip); end
  if isfield(sumdip, 'pow'),    dipmean.pow    = sumdip.pow   / length(dip); end
  if isfield(sumdip, 'coh'),    dipmean.coh    = sumdip.coh   / length(dip); end
  if isfield(sumdip, 'rv'),     dipmean.rv     = sumdip.rv    / length(dip); end
  if isfield(sumdip, 'noise'),  dipmean.noise  = sumdip.noise / length(dip); end
  if isfield(sumdip, 'nai'),    dipmean.nai    = sumdip.nai   / length(dip); end
  % for the cell array with mom, this is done further below
  % for the cell array with csd, this is done further below

  % the estimates for variance and SEM are biased if we are working with the jackknife/bootstrap
  % determine the proper variance scaling that corrects for this bias
  % note that Ntrials is not always the same as the length of dip, especially in case of the bootstrap
  if strcmp(source.method, 'singletrial')
    bias = 1;
  elseif strcmp(source.method, 'rawtrial')
    bias = 1;
  elseif strcmp(source.method, 'jackknife')
    % Effron gives SEM estimate for the jackknife method in equation 11.5 (paragraph 11.2)
    % to get the variance instead of SEM, we also have to multiply with the number of trials
    bias = (Ntrials - 1)^2;
  elseif strcmp(source.method, 'bootstrap')
    % Effron gives SEM estimate for the bootstrap method in algorithm 6.1 (equation 6.6)
    % to get the variance instead of SEM, we also have to multiply with the number of trials
    bias = Ntrials;
  elseif strcmp(source.method, 'pseudovalue')
    % note that I have not put any thought in this aspect yet
    warning('don''t know how to compute bias for pseudovalue resampling');
    bias = 1;
  end

  % compute the variance over all repetitions
  if isfield(sumdip, 'var'),    dipvar.var    = bias*(sqrdip.var    - (sumdip.var   .^2)/length(dip))/(length(dip)-1); end
  if isfield(sumdip, 'pow'),    dipvar.pow    = bias*(sqrdip.pow    - (sumdip.pow   .^2)/length(dip))/(length(dip)-1); end
  if isfield(sumdip, 'coh'),    dipvar.coh    = bias*(sqrdip.coh    - (sumdip.coh   .^2)/length(dip))/(length(dip)-1); end
  if isfield(sumdip, 'rv' ),    dipvar.rv     = bias*(sqrdip.rv     - (sumdip.rv    .^2)/length(dip))/(length(dip)-1); end
  if isfield(sumdip, 'noise' ), dipvar.noise  = bias*(sqrdip.noise  - (sumdip.noise .^2)/length(dip))/(length(dip)-1); end
  if isfield(sumdip, 'nai' ),   dipvar.nai    = bias*(sqrdip.nai    - (sumdip.nai   .^2)/length(dip))/(length(dip)-1); end

  % compute the SEM over all repetitions
  if isfield(sumdip, 'var'),    dipsem.var    = (dipvar.var   /Ntrials).^0.5; end
  if isfield(sumdip, 'pow'),    dipsem.pow    = (dipvar.pow   /Ntrials).^0.5; end
  if isfield(sumdip, 'coh'),    dipsem.coh    = (dipvar.coh   /Ntrials).^0.5; end
  if isfield(sumdip, 'rv' ),    dipsem.rv     = (dipvar.rv    /Ntrials).^0.5; end
  if isfield(sumdip, 'noise' ), dipsem.noise  = (dipvar.noise /Ntrials).^0.5; end
  if isfield(sumdip, 'nai' ),   dipsem.nai    = (dipvar.nai   /Ntrials).^0.5; end

  % compute the mean and SEM over all repetitions for the cell array with mom
  if isfield(dip(trial), 'mom')
    for i=1:length(dip(1).mom)
      dipmean.mom{i} = sumdip.mom{i}/length(dip);
      dipvar.mom{i} = bias*(sqrdip.mom{i} - (sumdip.mom{i}.^2)/length(dip))/(length(dip)-1);
      dipsem.mom{i} = (dipvar.mom{i}/Ntrials).^0.5;
    end
  end

  % compute the mean and SEM over all repetitions for the cell array with csd
  if isfield(dip(trial), 'csd')
    for i=1:length(dip(1).csd)
      dipmean.csd{i} = sumdip.csd{i}/length(dip);
      dipvar.csd{i} = bias*(sqrdip.csd{i} - (sumdip.csd{i}.^2)/length(dip))/(length(dip)-1);
      dipsem.csd{i} = (dipvar.csd{i}/Ntrials).^0.5;
    end
  end
  
  if strcmp(source.method, 'pseudovalue')
    % keep the trials, since they have been converted to pseudovalues
  else
    % remove the original trials
    source = rmfield(source, 'trial');
  end
  % assign the output source structure
  source.avg = dipmean;
  source.var = dipvar;
  source.sem = dipsem;
end

if strcmp(cfg.resolutionmatrix, 'yes')
  % this is only implemented for pcc and no refdips/chans at the moment
  Nchan        = size(source.leadfield{source.inside(1)}, 1);
  Ninside      = length(source.inside);
  allfilter    = zeros(Ninside,Nchan);
  allleadfield = zeros(Nchan,Ninside);
  dipsel       = match_str(source.avg.csdlabel, 'scandip');
  progress('init', cfg.feedback, 'computing resolution matrix');
  for diplop=1:length(source.inside)
    progress(diplop/length(source.inside), 'computing resolution matrix %d/%d\n', diplop, length(source.inside));
    i = source.inside(diplop);
    % concatenate all filters
    allfilter(diplop,:) = source.avg.filter{i}(dipsel,:);
    % concatenate all leadfields
    allleadfield(:,diplop) = source.leadfield{i};
  end
  progress('close');
  % multiply the filters and leadfields to obtain the resolution matrix
  % see equation 1 and 2 in De Peralta-Menendez RG, Gonzalez-Andino SL: A critical analysis of linear inverse solutions to the neuroelectromagnetic inverse problem. IEEE Transactions on Biomedical Engineering 45: 440-448, 1998.
  source.resolution = nan*zeros(Ndipole, Ndipole);
  source.resolution(source.inside, source.inside) = allfilter*allleadfield;
end

% FIXME this is a bit an odd place to do this, but otherwise the
% descriptive statistics are computed and the trial-field is thrown out
% the field should be added for consistency
if strcmp(cfg.singletrial, 'yes') && isfield(source.avg, 'csdlabel'),
  source.method = 'singletrial';
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: sourcedescriptives.m,v 1.29 2006/07/04 16:04:50 roboos Exp $';
% remember the configuration details of the input data
try, cfg.previous = source.cfg; end
% remember the exact configuration details in the output 
source.cfg = cfg;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% helper function to obtain the largest singular value or trace of the
% source CSD matrices resulting from DICS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function s = lambda1(x);
s = svd(x);
s = s(1);

